// Solution to problem C of eindwedstrijd Voorbereiding
// Programmeerwedstrijden: Canyon Crossing

#include <iostream>
using namespace std;
const int MAXR = 1000;
const int MAXC = 1000;
const int MAXSQUARES = 2000000;

int q0row[MAXSQUARES],
    q0col[MAXSQUARES],
    q1row[MAXSQUARES],
    q1col[MAXSQUARES],
    q0first, q0last,
    q1first, q1last,
    map[MAXR+1][MAXC+1],
    dist[MAXR+1][MAXC+1];
bool done[MAXR+1][MAXC+1];
    

//*************************************************************************

void addqueue (int qrow[], int qcol[], int row, int col, int &qlast)
{
  if (qlast==MAXSQUARES-1)
    qlast = 0;
  else
    qlast ++;
  qrow[qlast] = row;
  qcol[qlast] = col;

}  // addqueue

//*************************************************************************

// pre: qfirst <= qlast
void dequeue (int qrow[], int qcol[], int &row, int &col, int &qfirst)
{
  row = qrow[qfirst];
  col = qcol[qfirst];
  if (qfirst==MAXSQUARES-1)
    qfirst = 0;
  else
    qfirst++;

}  // dequeue

//*************************************************************************

void process (int row, int col, int minheight, int curbridges,
              int R, int C, int K)
{
  if (row>=1 && row<=R && col>=1 && col<=C)
  { if (!done[row][col])

    { if (map[row][col]>=minheight)  // we do not need extra bridge
      { if (curbridges<dist[row][col])
        { addqueue (q0row, q0col, row, col, q0last);
          dist[row][col] = curbridges;
        }
      }
      else  // we do need extra bridge
      { if (curbridges+1<dist[row][col] && curbridges+1<=K)
        { addqueue (q1row, q1col, row, col, q1last);
          dist[row][col] = curbridges+1;
        }
      }
    }

  }

}  // process

//*************************************************************************

void print (int R, int C, int K)
{ int row, col;

  cout << "Dist:" << endl;
  for (row=1;row<=R;row++)
  { for (col=1;col<=C;col++)
    { cout << ' ' << dist[row][col];
    }
    cout << endl;
  }  // for row

  cout << "Done:" << endl;
  for (row=1;row<=R;row++)
  { for (col=1;col<=C;col++)
    { cout << ' ' << (int)(done[row][col]);
    }
    cout << endl;
  }  // for row

}  // print

//*************************************************************************

int main ()
{ int R, C, K,
      curmin, curmax,  // min/maximum heights we consider
      row, col,
      minheight,
      contin,   // code for continuation of while-loop
      curbridges,  // current number of bridges for q0
      i;  // counter in q1

  cin >> R >> C >> K;

  curmax = 0;
  curmin = 1000000000;

  for (row=1;row<=R;row++)
    for (col=1;col<=C;col++)
    { cin >> map[row][col];
      if (map[row][col] > curmax)
        curmax = map[row][col];
      if (map[row][col] < curmin)
        curmin = map[row][col];
    }

  curmax++;  // it won't work with this height
             // it will work with curmin


  while (curmax - curmin >= 2)
  {
    minheight = (curmin+curmax) / 2;

    q0first = 0;
    q0last = -1;
    q1first = 0;
    q1last = -1;

    for (row=1;row<=R;row++)
      for (col=1;col<=C;col++)
      { done[row][col] = false;
        dist[row][col] = R*C+1;
      }

//    print (R, C, K);

    for (col=1;col<=C;col++)
    { if (map[1][col]>=minheight)
      { addqueue (q0row, q0col, 1, col, q0last);
        dist[1][col] = 0;
      }
      else  // we need one bridge
      { if (K>=1)
        { addqueue (q1row, q1col, 1, col, q1last);
          dist[1][col] = 1;
        }
      }
    }

//    print (R, C, K);

    contin = 0;
    curbridges = 0;
    while (contin==0)
    {
      if (q0first==q0last+1)  // q0 empty, so move q1 to q0
      { for (i=q1first;i<=q1last;i++)
          addqueue (q0row, q0col, q1row[i], q1col[i], q0last);
        q1first = 0;
        q1last = -1;
        curbridges ++;
      }

      if (q0first==q0last+1)  // q0 still empty
        contin = 1;  // means failed
      else  // q0 is not empty
      { dequeue (q0row, q0col, row, col, q0first);
        if (!done[row][col])  // not yet expanded
        { done[row][col] = true;
          if (row==R)  // arrived at other side
            contin = 2;  // means succeed
          else
          { process (row, col-1, minheight, curbridges, R, C, K);
            process (row, col+1, minheight, curbridges, R, C, K);
            process (row-1, col, minheight, curbridges, R, C, K);
            process (row+1, col, minheight, curbridges, R, C, K);
          }
        }  // if
      }  // else
    }  // while contin
       
//    print (R, C, K);

    if (contin==1)  // failed with minheight
      curmax = minheight;
    else
      curmin = minheight;

  }  // while curmax-curmin>=2
      
  cout << curmin << endl;
  
  return 0;
}
