// Solution to problem H of eindwedstrijd Voorbereiding
// Programmeerwedstrijden: Hexagonal Rooks

#include <iostream>
#include <cstring>
using namespace std;

const int MaxN = 91;  // all cells of the chess board
bool neighbour[MaxN][MaxN];   // adjacency matrix

//************************************************************************

// Compute height of column i.
// Pre: 0<=i<=10
int heightOfColumn (int i)
{
  if (i<=5)
    return (6+i);
  else
    return (16-i);

}  // heightOfColumn

//************************************************************************

// Compute cell number corresponding to certain file and rank.
// Cells are numbered column by column from left to right, and within
// a column bottom-up
// Pre: file and rank correspond to a valid cell
// * file=0 corresponds to 'a'
// * ranks are 1-based
int codeCell (int file, int rank)
{ int cell;

  cell = 0;

  for (int i=0;i<file;i++)  // column i
  {
    cell += heightOfColumn (i);
  }

  cell += rank-1;  // ranks are 1-based, so subtract 1

  return cell;

}  // codeCell

//************************************************************************

// Compute file and rank corresponding to certain cell number
// Pre: 0<=cell<91
void decodeCell (int cell, int &file, int &rank)
{ int i;

  i = 0;
  while (cell>=heightOfColumn (i))
  { cell -= heightOfColumn (i);
    i++;
  }

  file = i;
  rank = cell+1;  // ranks are 1-based, so add 1

}  // decodeCell

//************************************************************************

// Compute file, rank, and diagonals corresponding to certain cell number.
// Pre: 0<=cell<91
void decodeCellDiag (int cell, int &file, int &rank,
                               int &downdiag, int &updiag)
{
  decodeCell (cell, file, rank);

    // determine number of `downwards diagonal'
  if (file<=5)
    downdiag = rank;
  else
    downdiag = rank + (file - 5);

  // determine number of `upwards diagonal'
  if (file<5)
    updiag = rank + (5 - file);
  else
    updiag = rank;

}  // decodeCellDiag

//************************************************************************

// Fill adjacency matrix neighbour with correct values.
void fillNeighbour ()
{ int cell1, cell2,
      file1, rank1,
      file2, rank2,
      downdiag1, downdiag2,
      updiag1, updiag2;

    // initialize
  for (cell1=0;cell1<MaxN;cell1++)
  { for (cell2=0;cell2<MaxN;cell2++)
      neighbour[cell1][cell2] = false;
  }

  for (cell1=0;cell1<MaxN;cell1++)
  { decodeCellDiag (cell1, file1, rank1, downdiag1, updiag1);

    for (cell2=0;cell2<MaxN;cell2++)
    { if (cell1!=cell2)
      { decodeCellDiag (cell2, file2, rank2, downdiag2, updiag2);

        if (file1==file2 || downdiag1==downdiag2 || updiag1==updiag2)
          neighbour[cell1][cell2] = true;
      }
    }
  }

}  // fillNeighbour

//************************************************************************

// Compute file and rank from input string str.
// Pre: str represents a valid cell
void decodeString (char *str, int &file, int &rank)
{ int len,
      digit;

  file = str[0] - 'a';
  len = strlen (str);
  rank = 0;
  for (int i=1;i<len;i++)
  { digit = str[i] - '0';
    rank = rank*10 + digit;
  }

}  // decodeString

//************************************************************************

int main ()
{ char str1[4],
       str2[4];
  int file1, rank1,
      file2, rank2,
      cell1, cell2,
      counter;

  fillNeighbour ();

  cin >> str1 >> str2;

  decodeString (str1, file1, rank1);
  decodeString (str2, file2, rank2);

  cell1 = codeCell (file1, rank1);
  cell2 = codeCell (file2, rank2);

  counter = 0;
  for (int cell=0;cell<MaxN;cell++)
  {  // check if this cell is both adjacent to cell1 and cell2

    if (neighbour[cell1][cell] && neighbour[cell][cell2])
      counter ++;
  }

  cout << counter << endl;

  return 0;

}
